/*Basic Neural Net implementation based on Brad's 2/22/05 Lecture
by Daniel Iglesia*/

import com.cycling74.max.*;
import java.util.Arrays;

class NeuralNet extends MaxObject{
	float inputVals[] = new float[8];
	float weights[] = new float[18];
	float weightsShifted[] = new float[18];;
	float factor = (float)0.01;
	int inputSize = 8;

	public NeuralNet(){
		declareAttribute("factor"); 
		declareAttribute("inputSize", null, "setInputSize"); //setting inputSize calls "setInputSize"
		declareOutlets(new int[]{DataTypes.FLOAT});
	}
		
	public void list(Atom[] a) {
		for(int i = 0; i < inputSize; i++){//for the size of inputSize
			inputVals[i] = (a[i]).toFloat(); //copy incoming list as floats into InputVals - extra items are truncated - too few items crashes!
		}
	}
	
	public void reset(){//randomizes all the weights
		for(int i = 0; i < weights.length; i++){
			weights[i] = (float)(Math.random()-.5);
		}
	}
	
	public void setInputSize(int newSize){ //resizes some arrays and calls reset()
		inputSize=newSize;
		weights = new float[(inputSize*2)+2];
		weightsShifted = new float [(inputSize*2)+2];
		inputVals = new float [inputSize];
		reset();
	}
	
	public float computeNetValue(float currentWeights[]){
		float Avalue=0, Bvalue=0;
		for(int i=0;i<inputVals.length;i++){
			Avalue+=inputVals[i]*currentWeights[i*2];
			Bvalue+=inputVals[i]*currentWeights[i*2+1];
		}
		return (Avalue*currentWeights[inputVals.length*2])+(Bvalue*currentWeights[inputVals.length*2+1]);
	}
		
	public void trainIt(float desiredValue) {
		for(int i = 0; i < weightsShifted.length; i++) {
			//shift weights a bit
			weightsShifted[i] = weights[i] + ((float)(Math.random()-.5)*factor);
		}
		//if the net value with shifted weights is closer to the desired value than the unshifted weights
		if (Math.abs(desiredValue-computeNetValue(weightsShifted)) < (Math.abs(desiredValue-computeNetValue(weights)))){
			//Then update weights with the new weights
			for(int i = 0; i < weightsShifted.length; i++) {
				weights[i] = weightsShifted[i];
			}
		}
	}

	public void calcIt(){ 
		outlet(0, computeNetValue(weights));
	}
}


